package kube

import (
	"strings"
	"crypto/sha512"
	"crypto/sha256"
	"crypto/md5"
	"encoding/base64"
	"encoding/hex"
	"regexp"
	"list"
)

secret: [ID=_]: this={
	(#metadataName & {id: ID, data: this.data, disable: false}).out
}

configMap: {}
cronJob: {}
deployment: {}
ingress: {}
persistentVolumeClaim: {}
probe: {}
prometheusRule: {}
role: {}
roleBinding: {}
secret: {}
service: {}
serviceAccount: {}
statefulSet: {}

objectSets: [
	configMap,
	cronJob,
	deployment,
	ingress,
	persistentVolumeClaim,
	probe,
	prometheusRule,
	role,
	roleBinding,
	secret,
	service,
	serviceAccount,
	statefulSet,
]

objects: [ for v in objectSets for x in v {x}]

#metadataName: {
	// in parameters:
	id: string
	DATA=data: {[string]: string | bytes, ...}
	disable: bool
	// out parameter:
	if !disable {
		out: metadata: name: id + "-" + (#shortHash & {data: DATA}).out
	}
	if disable {
		out: metadata: name: id
	}
}

#shortHash: {
	// in parameter:
	data: {[string]: string | bytes, ...}
	_hashLen:       6
	_sortedKeys:    list.SortStrings([ for k, _ in data {k}])
	_joinedKV:      strings.Join([ for _, k in _sortedKeys {k + ":" + hex.Encode(data[k])}], "\n")
	_joinedKVBytes: '\(_joinedKV)'
	_md5:           md5.Sum(_joinedKVBytes)
	_sha256:        sha256.Sum256(_joinedKVBytes)
	_sha512:        sha512.Sum512(_joinedKVBytes)
	_base64:        base64.Encode(null, _sha512+_sha256+_md5)
	_base36:        regexp.ReplaceAll("[^a-z0-9]", _base64, "")
	_fragments:     strings.SplitN( _base36, "", _hashLen+1)
	_truncated:     strings.Join(_fragments[0:_hashLen], "")
	// out parameter:
	out: _truncated
}
